/***************************************************************************
 *   Copyright (C) 2008 by Daniel Calviño Sánchez                          *
 *   danxuliu@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include <kdebug.h>

#include "Step.h"
#include "Option.h"
#include "WaitFor.h"
#include "WaitForSignal.h"

//public:

void Step::setActive(bool active) {
    if (active) {
        setup();
    } else {
        tearDown();
    }

    QListIterator<WaitFor*> it(mWaitsFor);
    while (it.hasNext()) {
        it.next()->setActive(active);
    }
}

void Step::addOption(Option* option, QObject* receiver, const char* slot) {
    if (mOptions.contains(option)) {
        kWarning() << "Option " << option->name() << " already added in step " << mId;
        return;
    }

    QListIterator<Option*> it(mOptions);
    while (it.hasNext()) {
        if (it.next()->name() == option->name()) {
            kWarning() << "Option named " << option->name() << " already added in step " << mId;
            return;
        }
    }

    option->setParent(this);

    mOptions.append(option);

    WaitForSignal* waitFor = new WaitForSignal(option, SIGNAL(selected()));
    addWaitFor(waitFor, receiver, slot);

    mOptionsWaitsFor.append(waitFor);
}

void Step::addWaitFor(WaitFor* waitFor, QObject* receiver, const char* slot) {
    if (mWaitsFor.contains(waitFor)) {
        kWarning() << "Same WaitFor already added in step " << mId;
        return;
    }

    waitFor->setActive(false);

    waitFor->setParent(this);

    mWaitsFor.append(waitFor);
    connect(waitFor, SIGNAL(waitEnded(WaitFor*)), receiver, slot);
}

void Step::removeOption(Option* option) {
    if (!mOptions.contains(option)) {
        kWarning() << "Tried to remove an Option not added in step " << mId;
        return;
    }

    option->setParent(0);

    int index = mOptions.indexOf(option);
    mOptions.removeAt(index);

    WaitFor* waitFor = mOptionsWaitsFor[index];
    removeWaitFor(waitFor);

    mOptionsWaitsFor.removeAt(index);
}

void Step::removeWaitFor(WaitFor* waitFor) {
    if (!mWaitsFor.contains(waitFor)) {
        kWarning() << "Tried to remove a WaitFor not added in step " << mId;
        return;
    }

    waitFor->setActive(false);

    waitFor->setParent(0);

    mWaitsFor.removeAt(mWaitsFor.indexOf(waitFor));
    disconnect(waitFor, SIGNAL(waitEnded(WaitFor*)));
}
